@@ -92,6 +92,8 @@ extern u_char	*linebuf;
 
 #define	CISTREQ(x, y)		(cistrcmp((char *)(x), (char *)(y)) == 0)
 #define	CISTREQN(x, y, n)	(cistrncmp((char *)(x), (char *)(y), n) == 0)
+#define	STREQ(x, y)		(strcmp((char *)(x), (char *)(y)) == 0)
+#define	STREQN(x, y, n)		(strncmp((char *)(x), (char *)(y), n) == 0)
 #define TOKENLEN(t) ((t)->t_len > 0 ? (int) (t)->t_len : strlen((char *)(t)->t_pname))
 
 #if	defined(lint) && defined(putc)
