@@ -118,7 +118,7 @@ extern int	       ctlsticky __((char *spec_host,char *addr_host));
 
 /* diagnostic.c: */
 extern char           *notaryacct __((int rc, char *okstr));
-extern void 	       notaryreport __(());
+extern void 	       notaryreport __((char*,char*,char*));
 extern void	       diagnostic __(());
 
 /* emptyline.c: */
