@@ -1,38 +1,6 @@
 #include <stdio.h>
 #include <errno.h>
 
-#ifndef	DEBUG_FOPEN
-
-FILE *__fopen(filename, type)
-     char *filename, *type;
-{
-	return fopen(filename, type);
-}
-
-FILE *__freopen(filename,type,stream)
-     char *filename, *type;
-     FILE *stream;
-{
-	return freopen(filename,type,stream);
-}
-
-FILE *__fdopen(fd,type)
-     int fd;
-     char *type;
-{
-	return fdopen(fd,type);
-}
-
-int __fclose(stream)
-     FILE *stream;
-{
-	return fclose(stream);
-}
-
-
-#else
-
-
 static FILE *fileptrs[512];
 static int topfp  = 0;
 int calls_fopen   = 0;
@@ -176,7 +144,3 @@ int __fclose(stream)
 
 	return fclose(stream);
 }
-
-
-
-#endif
