@@ -20,6 +20,7 @@
 # include FCNTL_H
 #endif
 #include "libdb/search.h"
+#include "splay.h"
 
 #include "prototypes.h"
 
@@ -143,7 +144,8 @@ run_relation(argc, argv)
 	struct db_info proto_config, *dbip;
 	struct db_kind *dbkp;
 	struct spblk *spl;
-	int c, symid, errflg, set_cache_size;
+	int c, errflg, set_cache_size;
+	spkey_t symid;
 	memtypes oval;
 	char *cp, *dbtype;
 
@@ -583,7 +585,7 @@ db(dbname, key)
 	u_char *dbname, *key;
 {
 	register int i, j, k;
-	memtypes oval;
+	memtypes oval = stickymem;
 	struct conscell *l, *ll, *tmp;
 	struct spblk *spl;
 	struct db_info *dbip;
