@@ -12,16 +12,23 @@
 
 #include <netdb.h>
 
-/* do we have a normal nameserver? */
+/* ================ do we have a normal nameserver? ================ */
 #ifdef	TRY_AGAIN
+
 #ifdef	BSDTYPES_H
 #include BSDTYPES_H
 #endif	/* BSDTYPES_H */
+
 #include <netinet/in.h>
+
 #ifdef NOERROR
 #undef NOERROR /* On Solaris 2.3 the  netinet/in.h  includes
 		  sys/stream.h, which has DIFFERENT "NOERROR" in it.. */
 #endif
+
+#ifdef linux
+#define __USE_BSD 1	/* Linux headers ... Auch..  <endian.h> */
+#endif
 #include <arpa/nameser.h>
 #include <resolv.h>
 #include "search.h"
@@ -292,18 +299,18 @@ getmxrr(host, localhost, ttlp, depth)
 	  h_errno = TRY_AGAIN;
 	  return NULL;
 	}
-	qlen = res_mkquery(QUERY, host, C_IN, T_MX, (char *)NULL, 0, NULL,
-			   (char *)&buf, sizeof(buf));
+	qlen = res_mkquery(QUERY, host, C_IN, T_MX, NULL, 0, NULL,
+			   (unsigned char *)&buf, sizeof(buf));
 	if (qlen < 0) {
 		fprintf(stderr, "search_res: res_mkquery (%s) failed\n", host);
 		strcpy(h_errhost, host);	/* use strcat on purpose */
 		h_errno = NO_RECOVERY;
 		return NULL;
 	}
-	n = res_send((char *)&buf, qlen, (char *)&answer, sizeof(answer));
+	n = res_send((void *)&buf, qlen, (unsigned char *)&answer, sizeof(answer));
 	if (n < 0) {
 	  /* Retry it ONCE.. */
-	  n = res_send((char *)&buf, qlen, (char *)&answer, sizeof(answer));
+	  n = res_send((void *)&buf, qlen, (unsigned char *)&answer, sizeof(answer));
 	  if (n < 0) {
 	    if (D_bind || _res.options & RES_DEBUG)
 	      fprintf(stderr,
@@ -368,7 +375,7 @@ getmxrr(host, localhost, ttlp, depth)
 	maxpref = -1;
 	/* assert: stickymem == MEM_MALLOC;  for storing RHS of MX RR's */
 	while (--ancount >= 0 && cp < eom && nmx < (sizeof mx/sizeof mx[0])) {
-		n = dn_expand((msgdata *)&answer, eom, cp, hbuf, sizeof hbuf);
+		n = dn_expand((msgdata *)&answer, eom, cp, (void*)hbuf, sizeof hbuf);
 		if (n < 0)
 			break;
 		cp += n;
@@ -381,7 +388,7 @@ getmxrr(host, localhost, ttlp, depth)
 		cp += sizeof(u_short);				/* dlen */
 		if (type == T_CNAME) {
 			cp += dn_expand((msgdata *)&answer, eom, cp,
-					realname, sizeof realname);
+					(void*)realname, sizeof realname);
 			continue;
 		} else if (type != T_MX)  {
 			if (D_bind || _res.options & RES_DEBUG)
@@ -394,7 +401,7 @@ getmxrr(host, localhost, ttlp, depth)
 		mx[nmx].ttl = ttl;
 		mx[nmx].pref = _getshort(cp);
 		cp += sizeof(u_short);		/* MX preference value */
-		n = dn_expand((msgdata *)&answer, eom, cp, hbuf, sizeof hbuf);
+		n = dn_expand((msgdata *)&answer, eom, cp, (void*)hbuf, sizeof hbuf);
 		if (n < 0)
 			break;
 		cp += n;
@@ -466,7 +473,8 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtype with completion */
 	int depth;
 {
 	struct conscell *rval;
-	char buf[BUFSIZ], **domain;
+	char buf[BUFSIZ], *domain;
+	int i;
 
 	if (depth > 4) {
 		fprintf(stderr,
@@ -483,8 +491,9 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtype with completion */
 		if (*host != '\0' && *(host+strlen(host)-1) == '.')
 			return NULL;
 	}
-	for (domain = _res.dnsrch; h_errno == 0 && *domain != NULL; domain++) {
-		sprintf(buf, "%s.%s", host, *domain);
+	for (i = 0, domain = _res.dnsrch[i];
+	     h_errno == 0 && (domain = _res.dnsrch[i]) != NULL; ++i) {
+		sprintf(buf, "%s.%s", host, domain);
 		if ((rval = getrrtype(buf, rrtype, ttlp, depth+1)) != NULL)
 			return rval;
 	}
@@ -520,8 +529,8 @@ getrrtype(host, rrtype, ttlp, depth)
 		return NULL;
 	}
 
-	qlen = res_mkquery(QUERY, host, C_IN, rrtype, (char *)NULL, 0, NULL,
-			(char *)&buf, sizeof(buf));
+	qlen = res_mkquery(QUERY, host, C_IN, rrtype, NULL, 0, NULL,
+			   (unsigned char *)&buf, sizeof(buf));
 	if (qlen < 0) {
 		if (D_bind || _res.options & RES_DEBUG)
 			fprintf(stderr,
@@ -530,10 +539,10 @@ getrrtype(host, rrtype, ttlp, depth)
 		h_errno = NO_RECOVERY;
 		return NULL;
 	}
-	n = res_send((char *)&buf, qlen, (char *)&answer, sizeof(answer));
+	n = res_send((void *)&buf, qlen, (unsigned char *)&answer, sizeof(answer));
 	if (n < 0) {
 	  /* Retry it ONCE.. */
-	  n = res_send((char *)&buf, qlen, (char *)&answer, sizeof(answer));
+	  n = res_send((void*)&buf, qlen, (unsigned char *)&answer, sizeof(answer));
 	  if (n < 0) {
 	    if (D_bind || _res.options & RES_DEBUG)
 	      fprintf(stderr,
@@ -562,7 +571,7 @@ getrrtype(host, rrtype, ttlp, depth)
 			if (qdcount > 0 && strchr(host, '.') == NULL) {
 				cp = (msgdata *)&answer + sizeof(HEADER);
 				if (dn_expand((msgdata *)&answer,eom,cp,
-					      (u_char*)hb,sizeof hb)>=0) {
+					      (void*)hb,sizeof hb)>=0) {
 					if (hb[0] == '\0') {
 						hb[0] = '.'; hb[1] = '\0';
 					}
@@ -587,7 +596,7 @@ getrrtype(host, rrtype, ttlp, depth)
 	maxttl = 0;
 	l = NULL;
 	for (lhead = NULL; --ancount >= 0 && cp < eom; cp = nextcp) {
-		n = dn_expand((msgdata *)&answer, eom, cp, nbuf, sizeof(nbuf));
+		n = dn_expand((msgdata *)&answer, eom, cp, (void*)nbuf, sizeof(nbuf));
 		if (n < 0)
 			break;
 		if (first) {
@@ -695,7 +704,7 @@ getrrtype(host, rrtype, ttlp, depth)
 				continue;
 
 			if ((n = dn_expand((msgdata *)&answer, eom, cp,
-					   nbuf, sizeof(nbuf))) < 0)
+					   (void*)nbuf, sizeof(nbuf))) < 0)
 				break;
 
 			if (type == T_CNAME && rrtype != T_ANY &&
