@@ -1,16 +1,26 @@
 /* Various prototypes for ROUTER of Zmailer */
 
 #ifndef	__
-#ifdef __STDC__
-#define __(x) x
-#else
-#define __(x) ()
-#define const
-#define volatile
+# ifdef __STDC__
+#  define __(x) x
+# else
+#  define __(x) ()
+#  define const
+#  define volatile
+# endif
 #endif
+
+#ifdef  USE_SETUIDX	/* IBM AIXism */
+# include <sys/id.h>
+# ifdef USE_SETREUID
+#  define  setreuid(x,y) setuidx(ID_REAL|ID_EFFECTIVE, y)
+# else /* !USE_SETREUID */
+#  define  setuid(y)     setuidx(ID_REAL|ID_EFFECTIVE, y)
+# endif
 #endif
 
 /* various libraries, standard libc included.. */
+
 extern char	*erraddrlog;
 extern char	*gs_name;
 extern char	*monthname[];
