@@ -243,10 +243,10 @@ main(argc, argv)
 			killprevious(-SIGTERM, pidfile);
 		if (nrouters > 1) {
 			int pgrp, ppid = getpid();
-#ifdef	USE_BSDSETPGRP
+#ifdef	USE_BSDGETPGRP
 			pgrp = getpgrp(ppid);	/* BSD  -style */
 #else
-			pgrp = getpgrp();	/* SysV -style */
+			pgrp = getpgrp();	/* SysV/POSIX -style */
 #endif
 			if (pgrp != ppid) {
 				fprintf(stderr, "process group %d != pid %d\n",
