@@ -2,6 +2,10 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	Lots of modifications (new guts, more or less..) by
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ */
 
 /*
  * ZMailer transport scheduler.
@@ -50,7 +54,7 @@ char	*progname;
 char	*postoffice;
 char	*rendezvous;
 char	*pidfile = PID_SCHEDULER;
-char	*mailshare, *mailbin;
+char	*mailshare;
 char	*log;
 static int mustexit;
 static int gotalarm;
@@ -65,7 +69,8 @@ int	hungry_childs = 0;
 static int vtxprep_skip      = 0;
 static int vtxprep_skip_any  = 0;
 static int vtxprep_skip_lock = 0;
-static time_t next_dirscan   = 0;
+static time_t next_dirscan     = 0;
+static time_t next_idlecleanup = 0;
 
 #include "memtypes.h"
 extern memtypes stickymem;
@@ -75,7 +80,7 @@ static struct ctlfile *vtxprep __((struct ctlfile *));
 static int  vtxfill __((struct vertex *, struct config_entry *));
 static void link_in __((int flag, struct vertex *vp, u_char *s));
 static int  lockverify __((struct ctlfile *, char *));
-static int  match   __((char *, char*));
+static int  globmatch   __((char *, char*));
 static void vtxdo   __((struct vertex *, struct config_entry *, char *));
 
 static SIGNAL_TYPE sig_exit   __((int sig));
@@ -120,6 +125,7 @@ static int dirqueuescan __((const char *dir, struct dirqueue *dq));
 static int syncweb __((struct dirqueue *dq, struct config_entry *ce));
 
 int global_maxkids = 1000;
+time_t now;
 
 int
 main(argc, argv)
@@ -130,16 +136,13 @@ main(argc, argv)
 	char *config, *cp;
 	int i, daemonflg, c, errflg, version, fd;
 	long offout, offerr;
-	time_t now;
 	struct config_entry *tp;
 	struct dirqueue dqb;
 	struct dirqueue *dq = &dqb;
+	int freeze = 0;
 
 	/* setlinebuf(stderr);  -- no need for this ? */
 
-fprintf(stderr,"*** THIS IS ZMAILER'S NEW SCHEDULER, AND IS NOT YET USABLE ***\n");
-exit(64);
-
 	memset(&dqb,0,sizeof(dqb));
 
 	if ((progname = strrchr(argv[0], '/')) == NULL)
@@ -152,7 +155,7 @@ exit(64);
 	resources_maximize_nofiles();
 
 	/* The theory is, that scheduler needs circa 10 fd's for its own uses,
-	   and it will use all others on child-process return channels. */
+	   and it will use all others on child-process communication fifos. */
 
 	global_maxkids = resources_query_nofiles()-10;
 
@@ -160,11 +163,14 @@ exit(64);
 	daemonflg = 1;
 	dlyverbose = 0;
 	verbose = errflg = version = 0;
-	while ((c = getopt(argc, argv, "disvf:L:N:P:Q:VW")) != EOF)
+	while ((c = getopt(argc, argv, "divf:FL:N:P:Q:VW")) != EOF)
 		switch (c) {
 		case 'f':	/* override default config file */
 			config = optarg;
 			break;
+		case 'F':
+			freeze = 1;
+			break;
 		case 'L':	/* override default log file */
 			log = optarg;
 			break;
@@ -192,8 +198,6 @@ exit(64);
 		case 'i':	/* interactive */
 			daemonflg = 0;
 			break;
-		case 's':
-			break;
 		case 'V':
 			version = 1;
 			daemonflg = 0;
@@ -211,8 +215,6 @@ exit(64);
 	}
 	if ((mailshare = getzenv("MAILSHARE")) == NULL)
 	  mailshare = MAILSHARE;
-	if ((mailbin = getzenv("MAILBIN")) == NULL)
-	  mailbin = MAILBIN;
 	if ((cp = getzenv("LOGDIR")) != NULL)
 	  qlogdir = cp;
 	if (daemonflg && log == NULL) {
@@ -231,13 +233,17 @@ exit(64);
 	  setbuf(stderr,NULL);	/* [mea] No buffering on stdout.. */
 	}
 #ifdef USE_SIGREAPER
-#ifdef SIGCLD
+# ifdef SIGCLD
 	SIGNAL_HANDLE(SIGCLD,  sig_chld);
-	/* SIGNAL_HANDLE(SIGCLD,SIG_IGN); */	/* Auto-reap the kids.. */
-#else
+# else
 	SIGNAL_HANDLE(SIGCHLD, sig_chld);
-	/* SIGNAL_HANDLE(SIGCHLD,SIG_IGN); */
-#endif
+# endif
+#else
+# ifdef SIGCLD
+	SIGNAL_HANDLE(SIGCLD,SIG_IGN);		/* Auto-reap the kids.. */
+# else
+	SIGNAL_HANDLE(SIGCHLD,SIG_IGN);
+# endif
 #endif
 
 #ifdef	SIGUSR1
@@ -284,7 +290,7 @@ exit(64);
 	    /* NOTREACHED */
 	  }
 	  detach();		/* leave worldy matters behind */
-	  now = time(NULL);
+	  time(&now);
 	  printf("%s: scheduler daemon (%s)\n\tstarted at %s\n",
 		 progname, Version, (char *)rfc822date(&now));
 	}
@@ -340,8 +346,8 @@ exit(64);
 #ifdef	MALLOC_TRACE
 	mal_leaktrace(1);
 #endif	/* MALLOC_TRACE */
-	while (!mustexit) {
-	  time_t now, timeout;
+	do {
+	  time_t timeout;
 
 	  time(&now);
 
@@ -353,7 +359,12 @@ exit(64);
 	    dirqueuescan(".", dq);
 	  }
 
-	  /* Submit one item from pre-scheduler-queue into scheduler */
+	  if (now >= next_idlecleanup) {
+	    next_idlecleanup = now + 20; /* 20 second interval */
+	    idle_cleanup();
+	  }
+
+	  /* Submit one item from pre-scheduler-queue into the scheduler */
 	  if (dq->wrkcount > 0)
 	    syncweb(dq, tp);
 
@@ -364,8 +375,8 @@ exit(64);
 	  if (dq->wrkcount > 0)
 	    timeout = now;
 
-	  /* Submit possible new jobs */
-	  if (doagenda() != 0)
+	  /* Submit possible new jobs (unless frozen) */
+	  if (!freeze && doagenda() != 0)
 	    timeout = now;	/* we still have some jobs avail for start */
 
 	  gotalarm = 0;
@@ -382,8 +393,8 @@ exit(64);
 	      rereadcf = 0;
 	    }
 	  } while (mux(timeout) == 0 && !gotalarm &&
-		   dq->wrkcount == 0);
-	}
+		   dq->wrkcount == 0 && !mustexit);
+	} while (!mustexit);
 #ifdef	MALLOC_TRACE
 	mal_dumpleaktrace(stderr);
 #endif	/* MALLOC_TRACE */
@@ -580,7 +591,7 @@ static int syncweb(dq, cehead)
 	dq->wrkcount -= 1;
 	file  =   dq->stats[dq->wrkcount]->name;
 	stbuf = &(dq->stats[dq->wrkcount]->st);
-	ino   =   dq->stats[dp->wrkcount]->ino;
+	ino   =   dq->stats[dq->wrkcount]->ino;
 	/* Now we have pointers */
 
 	/* Sometimes we may get files already in processing
@@ -747,7 +758,7 @@ slurp(fd, ino)
 	/* copy over the offsets */
 	while (--i >= 0)
 	  *lp++ = *ip++;
-	cfp->id  = ino;
+	cfp->id = ino;
 	cfp->mid = NULL;
 	free(offset);	/* release the block */
 	return cfp;
@@ -1088,31 +1099,30 @@ static struct ctlfile *vtxprep(cfp)
 
 	  *ehost = '\0';
 	  /* compare with the last ones */
-	  if ((i+1) == opcnt || /* last one, or different from previous one */
-	      cistrcmp(channel, l_channel) || cistrcmp(host, l_host)) {
+	  if (cistrcmp(channel, l_channel) || cistrcmp(host, l_host)) {
 	    /* wrap and tie the old vertex node */
 	    if (i != svn) {
 	      u_int alloc_size = (u_int) (sizeof (struct vertex) +
 					  (i - svn - 1) * sizeof (int));
 	      vp = (struct vertex *)emalloc(alloc_size);
 	      memset((char*)vp, 0, alloc_size);
-	      vp->cfp        = cfp;
-	      vp->linkcnt    = 0;
+	      vp->cfp          = cfp;
+	      vp->linkcnt      = 0;
 	      defaultconfigentry(&vp->ce,NULL);
 	      vp->next[L_CTLFILE] = NULL;
 	      vp->prev[L_CTLFILE] = pvp;
-	      vp->message    = NULL;
-	      vp->retryindex = 0;
-	      vp->nextitem   = NULL;
-	      vp->previtem   = NULL;
-	      vp->wakeup     = 0;
-	      vp->proc       = NULL;
-	      vp->ngroup     = i - svn;
-	      vp->attempts   = 0;
-	      vp->notary     = NULL;
-	      vp->sender     = offarr[svn].sender;
-	      vp->headeroffset = offarr[i].headeroffset; /* They are the same*/
-	      vp->drptoffset   = offarr[i].drptoffset;
+	      vp->message      = NULL;
+	      vp->retryindex   = 0;
+	      vp->nextitem     = NULL;
+	      vp->previtem     = NULL;
+	      vp->wakeup       = 0;
+	      vp->proc         = NULL;
+	      vp->ngroup       = i - svn;
+	      vp->attempts     = 0;
+	      vp->notary       = NULL;
+	      vp->sender       = offarr[svn].sender;
+	      vp->headeroffset = offarr[svn].headeroffset; /*They are similar*/
+	      vp->drptoffset   = offarr[svn].drptoffset;
 	      while (svn < i) {
 		vp->index[i-svn-1] = offarr[svn].myidx;
 		++svn;
@@ -1120,7 +1130,7 @@ static struct ctlfile *vtxprep(cfp)
 	      *pvpp = vp;
 	      pvpp = &vp->next[L_CTLFILE];
 	      pvp = vp;
-	      link_in(L_HOST, vp, (u_char *)l_host);
+	      link_in(L_HOST,    vp, (u_char *)l_host);
 	      link_in(L_CHANNEL, vp, (u_char *)l_channel);
 	    }
 	    /* create a new vertex node */
@@ -1135,6 +1145,40 @@ static struct ctlfile *vtxprep(cfp)
 	  l_host      = host;
 	} /* for( .. i < opcnt .. ) */
 
+	/* wrap and tie the old vertex node (this is a copy of code above) */
+	if (i != svn) {
+	  u_int alloc_size = (u_int) (sizeof (struct vertex) +
+				      (i - svn - 1) * sizeof (int));
+	  vp = (struct vertex *)emalloc(alloc_size);
+	  memset((void*)vp, 0, alloc_size);
+	  vp->cfp = cfp;
+	  vp->linkcnt  = 0;
+	  defaultconfigentry(&vp->ce,NULL);
+	  vp->next[L_CTLFILE] = NULL;
+	  vp->prev[L_CTLFILE] = pvp;
+	  vp->message = NULL;
+	  vp->retryindex = 0;
+	  vp->nextitem = NULL;
+	  vp->previtem = NULL;
+	  vp->proc = NULL;
+	  vp->wakeup = 0;
+	  vp->ngroup = i - svn;
+	  vp->attempts = 0;
+	  vp->notary = NULL;
+	  vp->sender = offarr[i].sender;
+	  vp->headeroffset = offarr[svn].headeroffset; /* Just any of them will do */
+	  vp->drptoffset = offarr[svn].drptoffset;
+	  while (svn < i) {
+	    vp->index[i-svn-1] = offarr[svn].myidx;
+	    ++svn;
+	  }
+	  *pvpp = vp;
+	  pvpp = &vp->next[L_CTLFILE];
+	  pvp = vp;
+	  link_in(L_HOST, vp, (u_char *)host);
+	  link_in(L_CHANNEL, vp, (u_char *)channel);
+	}
+
 	*l_echannel = *l_ehost = ' ';
 	/*
 	   for (vp = head; vp != NULL; vp = vp->next[L_CTLFILE]) {
@@ -1159,16 +1203,15 @@ static int vtxfill(vp, tp)
 {
 	/* if the channel doesn't match, there's no hope! */
 	if (verbose>1)
-	  printf("ch? %s %s\n",
-		 vp->orig[L_CHANNEL]->name, tp->channel);
+	  printf("ch? %s %s\n", vp->orig[L_CHANNEL]->name, tp->channel);
 	if (tp->channel[0] == '*' && tp->channel[1] == '\0')
 	  return 0; /* Never match the defaults entry! */
-	if (!match(tp->channel, vp->orig[L_CHANNEL]->name))
+	if (!globmatch(tp->channel, vp->orig[L_CHANNEL]->name))
 	  return 0;
 
 	if (!(tp->host == NULL || tp->host[0] == '\0' ||
 	      (tp->host[0] == '*' && tp->host[1] == '\0'))) {
-	  if (!match(tp->host, vp->orig[L_HOST]->name))
+	  if (!globmatch(tp->host, vp->orig[L_HOST]->name))
 	    return 0;
 	}
 
@@ -1176,6 +1219,7 @@ static int vtxfill(vp, tp)
 	  printf("host %s %s\n", vp->orig[L_HOST]->name, tp->host);
 
 	if (tp->interval != -1) vp->ce.interval = tp->interval;
+	if (tp->idlemax  != -1) vp->ce.idlemax  = tp->idlemax;
 	if (tp->expiry   != -1) vp->ce.expiry   = tp->expiry;
 	vp->ce.expiryform     = tp->expiryform;
 	vp->cfp->deliveryform = tp->deliveryform;
@@ -1183,7 +1227,7 @@ static int vtxfill(vp, tp)
 	if (tp->gid      != -1) vp->ce.gid = tp->gid;
 	if (tp->maxkids  != -1) vp->ce.maxkids = tp->maxkids;
 	if (tp->maxkidChannel != -1) vp->ce.maxkidChannel = tp->maxkidChannel;
-	if (tp->maxkidHost    != -1) vp->ce.maxkidHost    = tp->maxkidHost;
+	if (tp->maxkidThreads != -1) vp->ce.maxkidThreads = tp->maxkidThreads;
 	if (tp->nretries > 0) {
 	  vp->ce.nretries = tp->nretries;
 	  vp->ce.retries  = tp->retries;
@@ -1194,10 +1238,15 @@ static int vtxfill(vp, tp)
 	}
 	if (tp->bychannel != -1)
 	  vp->ce.bychannel = tp->bychannel;
-	if (tp->byhost != -1)
-	  vp->ce.byhost = tp->byhost;
+	else
+	  vp->ce.bychannel = 0;
+	if (tp->withhost != -1)
+	  vp->ce.withhost = tp->withhost;
+	else
+	  vp->ce.withhost = 0;
 	vp->ce.flags |= tp->flags;
 	vp->ce.mark   = vp->ce.pending = 0;
+	vp->ce.host   = tp->host;
 	if (tp->skew > 0) vp->ce.skew = tp->skew;
 	return 1;
 }
@@ -1232,10 +1281,8 @@ static void vtxdo(vp, cehead, path)
 	    break;
 	}
 	if (n == 0) {
-	  fprintf(stderr,
-		  "%s: no pattern matched %s/%s address\n",
-		  progname,
-		  vp->orig[L_CHANNEL]->name, vp->orig[L_HOST]->name);
+	  fprintf(stderr, "%s: no pattern matched %s/%s address\n",
+		  progname, vp->orig[L_CHANNEL]->name, vp->orig[L_HOST]->name);
 	  /* XX: memory leak here? */
 	  return;
 	}
@@ -1250,14 +1297,16 @@ static void vtxdo(vp, cehead, path)
 	  vp->ce.expiry = 0;
 	if (vp->ce.interval == -1)
 	  vp->ce.interval = 3600;
+	if (vp->ce.idlemax == -1)
+	  vp->ce.idlemax = vp->ce.interval * 3;
 	if (vp->ce.maxkids == -1)
 	  vp->ce.maxkids = global_maxkids;
 	if (vp->ce.maxkidChannel == -1)
 	  vp->ce.maxkidChannel = global_maxkids;
-	if (vp->ce.maxkidHost == -1)
-	  vp->ce.maxkidHost = global_maxkids;
-	if (vp->ce.byhost == -1)
-	  vp->ce.byhost = 0;
+	if (vp->ce.maxkidThreads == -1)
+	  vp->ce.maxkidThreads = global_maxkids;
+	if (vp->ce.withhost == -1)
+	  vp->ce.withhost = 0;
 	if (vp->ce.bychannel == -1)
 	  vp->ce.bychannel = 0;
 
@@ -1265,17 +1314,6 @@ static void vtxdo(vp, cehead, path)
 
 	if (verbose>1)
 	  vtxprint(vp);
-
-	if ((vp->ce.flags & FF_GANGSCHEDULE)
-	    && vp->prev[L_HOST]    != NULL
-	    && vp->prev[L_CHANNEL] != NULL
-	    && vp->ce.retries      == vp->prev[L_HOST]->ce.retries
-	    && vp->orig[L_HOST]    == vp->prev[L_HOST]->orig[L_HOST]
-	    && vp->orig[L_CHANNEL] == vp->prev[L_CHANNEL]->orig[L_CHANNEL]) {
-	  vp->retryindex = vp->prev[L_HOST]->retryindex;
-	  reschedule(vp, -(vp->prev[L_HOST]->wakeup), -1);
-	} else
-	  reschedule(vp, 0, -1);
 }
 
 
@@ -1298,7 +1336,7 @@ int vtxredo(spl)
 
 
 /* Shell-GLOB-style matching */
-static int match(pattern, string)
+static int globmatch(pattern, string)
 	register char	*pattern;
 	register char	*string;
 {
@@ -1307,7 +1345,7 @@ static int match(pattern, string)
 	  case '*':
 	    ++pattern;
 	    do {
-	      if (match(pattern, string))
+	      if (globmatch(pattern, string))
 		return 1;
 	    } while (*string++ != '\0');
 	    return 0;
@@ -1367,25 +1405,10 @@ static void link_in(flag, vp, s)
 	struct vertex *vp;
 	u_char *s;
 {
-	register u_char *cp;
-	struct web *wp;
-	struct spblk *spl;
+	struct web *wp = web_findcreate(flag,s);
+
+	wp->linkcnt += 1;
 
-	for (cp = s; *cp != '\0'; ++cp)
-	  if (isascii(*cp) && isupper(*cp))
-	    *cp = tolower(*cp);
-	spl = sp_lookup(symbol(s), spt_mesh[flag]);
-	if (spl == NULL || (wp = (struct web *)spl->data) == NULL) {
-	  wp = (struct web *)emalloc(sizeof (struct web));
-	  memset((void*)wp, 0, sizeof (struct web));
-	  sp_install(symbol(s), (u_char *)wp, 0, spt_mesh[flag]);
-	  wp->name = strsave((char *)s);
-	  wp->kids = 0;
-	  wp->link = NULL;
-	  wp->lastlink = NULL;
-	  wp->fifohead = NULL;
-	  wp->fifotail = NULL;
-	}
 	vp->next[flag] = NULL;
 	vp->prev[flag] = wp->lastlink;
 	vp->orig[flag] = wp;
@@ -1403,7 +1426,6 @@ char *
 timestring()
 {
 	static char timebuf[40];
-	time_t now;
 	struct tm *tp;
 
 	time(&now);
