@@ -2,6 +2,10 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	Lots of modifications (new guts, more or less..) by
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ */
 
 
 #include <stdio.h>
@@ -15,12 +19,14 @@
 #include <sys/file.h>
 #include NDIR_H
 #include "zmsignal.h"
+#include <stdlib.h>
+#include <unistd.h>
 
 #include "prototypes.h"
 
+
 static int  scheduler_nofiles = -1; /* Will be filled below */
-static void dotransport __((char **, int, struct vertex *, char*, char*));
-static void runcommand __((char **, int, char **, struct vertex *, struct web *, struct web*));
+static int  runcommand __((char **, struct vertex *, struct web *, struct web*));
 static void stashprocess __((int, int, int, struct web*, struct web*, struct vertex *));
 static void reclaim __((int, int));
 static void waitandclose __((int));
@@ -47,96 +53,62 @@ extern int errno;
 extern time_t time();
 
 
-/*
- * Collect up all the marked control file names (unmarking them in the process)
- * and send them to the command given by the parameter. Said command must be
- * forked off, and the appropriate structures set up to deal with the stdout
- * of that command.
- */
-
-static int t_n, t_nmax;
-static char **t_files;
-
-static int
-tscan(spl)
-	struct spblk *spl;
-{
-	struct ctlfile *cfp = (struct ctlfile *)spl->data;
-
-	/* assert cfp != NULL */
-
-	/* Is this marked ? */
-	if (cfp->mark == V_NONE)
-	  return -1; /* .. No. */
-
-	/* If the count of jobs exceeds our high guesses.. */
-	if (t_n >= t_nmax) {
-	  /* Report only once per scan.. */
-	  if (t_nmax > 0)
-	    printf("**AARGH**  MAXFILESPERTRANSPORT(%d) TURNED OUT TO BE TOO SMALL!\n", t_nmax);
-	  t_nmax = 0;
-	  /* Clear the markings.. */
-	  cfp->mark    = V_NONE;
-	  return -1;
-	}
-
-	t_files[t_n++] = cfp->mid;
-	cfp->mark    = V_NONE;
-	return 0;
-}
 
 void
-transport(vhead, channel, host)
-	struct vertex *vhead;
-	char *channel, *host;
+feed_child(proc)
+struct procinfo *proc;
 {
-	int i;
-	char *files[MAXFILESPERTRANSPORT];	/* XX */
+	char buf[500]; /* 60 or so SHOULD suffice.. */
+	int len, rc;
 
-	/* generate the file names we care about */
-	t_n     = 0;
-	t_nmax  = MAXFILESPERTRANSPORT;
-	t_files = files;
+	if (proc->vertex == NULL) return; /* Might be called without next process.. */
 
-	if (verbose)
-	  printf("transport(vhead,chan=\"%s\",host=\"%s\")\n",channel,host);
+	if (proc->hungry) --hungry_childs;
 
-	sp_scan(tscan, (struct spblk *)NULL, spt_mesh[L_CTLFILE]);
-	dotransport(files, t_n, vhead, channel, host);
+	proc->vertex->proc = proc; /* Flag that it is in processing */
+
+	if (proc->thread->thgrp->withhost)
+	  sprintf(buf, "%s\n", proc->vertex->cfp->mid);
+	else
+	  sprintf(buf, "%s\t%s\n", proc->vertex->cfp->mid, proc->ho->name);
+
+	len = strlen(buf);
+
+	rc = write(proc->tofd, buf, len);
+	if (rc != len) {
+	  /* Failed ?  How come ?
+	     Close the 'tofd' pipe, and clear proc ptr. */
+	  close(proc->tofd);
+	  proc->tofd = -1;
+	  proc->vertex->proc = NULL;
+	  return;
+	}
+
+	/* It is fed, clear this flag.. */
+	proc->hungry = 0;
 }
 
-static void dotransport(files, n, vhead, channel, host)
-	char *files[];
-	int n;
+/*
+ * start_child() -- build argv[], and do other inits for fork()ing
+ *                  and execve()ing a new transport program for us.
+ */
+
+int
+start_child(vhead, chwp, howp)
 	struct vertex *vhead;
-	char *channel, *host;
-{
-	struct spblk *spl;
 	struct web *chwp, *howp;
+{
 	char	*av[30], *s, *os, *cp, *ocp, buf[MAXPATHLEN];
 	int	i;
 
-	if (n == 0)
-		return;
-	chwp = NULL;
-	if (channel != NULL) {
-	  spl = sp_lookup(symbol((u_char *)channel), spt_mesh[L_CHANNEL]);
-	  if (spl == NULL || (chwp = (struct web *)spl->data) == NULL) {
-	    fprintf(stderr,"dotransport: channel='%s' not on internal database!\n",channel);
-	    return;		/* yes, this can happen */
-	  }
-	}
-	howp = NULL;
-	if (host != NULL) {
-	  spl = sp_lookup(symbol((u_char *)host), spt_mesh[L_HOST]);
-	  if (spl == NULL || (howp = (struct web *)spl->data) == NULL) {
-	    fprintf(stderr,"dotransport: host='%s' not on internal database!\n",host);
-	    return;		/* yes, this can happen */
-	  }
-	}
+	if (verbose)
+	  printf("transport(vhead,chan=\"%s\",host=\"%s\")\n",
+		 chwp->name,howp->name);
+
 	if (vhead->ce.argv == NULL) {
-	  fprintf(stderr, "No command defined for %s/%s!\n", channel, host);
-	  return;
+	  fprintf(stderr, "No command defined for %s/%s!\n",
+		  chwp->name, howp->name);
+	  return 0;
 	}
 	/*
 	 * Replace the $host and $channel strings in the command line.
@@ -144,9 +116,9 @@ static void dotransport(files, n, vhead, channel, host)
 	os = buf;
 	for (i = 0; vhead->ce.argv[i] != NULL; ++i) {
 	  if (cistrcmp(vhead->ce.argv[i], replhost) == 0) {
-	    av[i] = host;
+	    av[i] = howp->name;
 	  } else if (cistrcmp(vhead->ce.argv[i], replchannel) == 0) {
-	    av[i] = channel;
+	    av[i] = chwp->name;
 	  } else if (strchr(vhead->ce.argv[i], '$') != NULL) {
 	    s = os;
 	    for (cp = vhead->ce.argv[i]; *cp != '\0'; ++cp) {
@@ -172,46 +144,24 @@ static void dotransport(files, n, vhead, channel, host)
 	    av[i] = vhead->ce.argv[i];
 	}
 	av[i] = NULL;
-	/*
-	 * randomize the file names if appropriate, this is a simpleminded
-	 * but probably effective way of avoiding a single message blocking
-	 * some queue.  XX: This should really be expanded to invoke a general
-	 * strategy routine/mechanism.
-	 */
-	if (n > 1) {
-	  char *temp;
-	  u_int ni;
 
-	  for (i = 0; i < n; ++i) {
-	    ni = ranny(n-1);
-	    temp = files[i];
-	    files[i] = files[ni];
-	    files[ni] = temp;
-	  }
-	}
 	/* fork off the appropriate command with the appropriate stdin */
 	if (verbose) {
 	  printf("$");
 	  for (i = 0; av[i] != NULL; ++i)
 	    printf(" %s", av[i]);
-	  printf(" <<\\EOF\n");
-	  for (i = 0; i < n; ++i)
-	    printf("%s\n", files[i]);
-	  printf("\\EOF\n");
+	  printf("\n");
 	}
-	runcommand(files, n, av, vhead, chwp, howp);
+	return runcommand(av, vhead, chwp, howp);
 }
 
-static void runcommand(filearr, nelem, argv, vhead, chwp, howp)
-	char	*filearr[];
-	int	nelem;
+static int runcommand(argv, vhead, chwp, howp)
 	char	*argv[];
 	struct vertex *vhead;
 	struct web *chwp, *howp;
 {
-	int	i, pid, to[2], tofd, from[2], uid, gid;
+	int	i, pid, to[2], from[2], uid, gid;
 	char	*cmd;
-	FILE	*fp;
 	static int pipesize = 0;
 
 
@@ -219,24 +169,20 @@ static void runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	gid = vhead->ce.gid;
 	cmd = vhead->ce.command;
 
-	if (nelem <= 0)
-	  return;
 	if (epipe(to) < 0)
-	  return;
+	  return 0;
 	if (epipe(from) < 0) {
 	  close(to[0]);
 	  close(to[1]);
-	  return;
+	  return 0;
 	}
 	if (pipesize == 0)
 	  pipesize = resources_query_pipesize(to[0]);
-	tofd = dup(to[1]);
 
 	if (verbose)
 	  fprintf(stderr, "to %d/%d from %d/%d\n",
 		  to[0],to[1],from[0],from[1]);
 	if ((pid = fork()) == 0) {	/* child */
-	  int topfileno;
 	  if (to[0] != 0) {
 	    dup2(to[0],0);	/* child stdin */
 	    close(to[0]);
@@ -267,50 +213,30 @@ static void runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	  close(to[0]); close(to[1]);
 	  close(from[0]); close(from[1]);
 	  fprintf(stderr, "Fork failed!\n");
-	  return;
-	} else {		/* parent */
-	  close(to[0]);
-	  close(from[1]);
-	  ++numkids;
-	  if (chwp != NULL) chwp->kids += 1;
-	  if (howp != NULL) howp->kids += 1;
-	  /* save from[0] away as a descriptor to watch */
-	  stashprocess(pid, from[0], tofd, chwp, howp, vhead);
-	  /* Write the list of files into to[1], and close it.
-	     If dup() failed, we don't have channel for latter use. */
-	  if ((fp = fdopen(to[1], "w")) == NULL) {
-	    if (verbose)
-	      fprintf(stderr, "cannot open fd %d for w.\n",
-		      to[1]);
-	    close(to[1]);
-	    close(from[0]);
-	    return;
-	  }
-	  if (tofd < 0) { /* Old-way -- fallback */
-	    int position;
-	    for (i = position = 0; i < nelem; ++i) {
-	      position += strlen(filearr[i]) + 1;
-	      if (position + 1 >= pipesize)
-		break;
-	      fprintf(fp, "%s\n", filearr[i]);
-	    }
-	    fputc('\n', fp);
-	    if (verbose)
-	      fprintf(stderr, "wrote %d bytes to fd %d\n",
-		      position+1, fileno(fp));
-	  }
-	  /* For the NEW-WAY we actually wait for the client to tell
-	     that it is hungry! */
-	  fclose(fp);	/* This will also close the to[1] fd */
+	  return 0;
 	}
+
+	/* parent */
+
+	close(to[0]);
+	close(from[1]);
+	++numkids;
+	if (chwp != NULL) chwp->kids += 1;
+	if (howp != NULL) howp->kids += 1;
+	/* save from[0] away as a descriptor to watch */
+	stashprocess(pid, from[0], to[1], chwp, howp, vhead);
+	/* We wait for the child to report "#hungry", then we feed it.. */
+	return 1;
 }
 
+
 static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	int pid, fromfd, tofd;
 	struct web *chwp, *howp;
 	struct vertex *vhead;
 {
 	int i;
+	struct procinfo *proc;
 
 	if (verbose)
 	  fprintf(stderr, "stashprocess(%d, %d, %d, %s, %s)\n",
@@ -325,15 +251,19 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	  while (--i >= 0)
 	    cpids[i].pid = 0;
 	}
-	cpids[fromfd].pid    = pid;
-	cpids[fromfd].reaped = 0;
-	cpids[fromfd].ch     = chwp;
-	cpids[fromfd].ho     = howp;
-	cpids[fromfd].vertices = vhead;
-	cpids[fromfd].tofd   = tofd;
-	vhead->proc = &cpids[fromfd];
-	cpids[fromfd].carryover = NULL;
-	cpids[fromfd].hungry = 1;
+	proc = &cpids[fromfd];
+	proc->next   = NULL;
+	proc->pid    = pid;
+	proc->reaped = 0;
+	proc->ch     = chwp;
+	proc->ho     = howp;
+	proc->vertex = vhead;
+	proc->thread = vhead->thread;
+	vhead->thread->proc = proc;
+	proc->tofd   = tofd;
+	vhead->proc  = proc;
+	proc->carryover = NULL;
+	proc->hungry = 0;
 #ifdef O_NONBLOCK /* POSIXy thing */
 	/* set up non-blocking I/O */
 	i = fcntl(fromfd, F_GETFL, 0);
@@ -354,35 +284,33 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 static void reclaim(fromfd, tofd)
 	int fromfd, tofd;
 {
-	struct vertex *vp, *nvp;
+	struct vertex *vp, *nvp = NULL;
+	struct procinfo *proc = &cpids[fromfd];
 
-	cpids[fromfd].pid = 0;
-	cpids[fromfd].reaped = 0;
-	if (cpids[fromfd].carryover != NULL) {
+if (verbose)
+  fprintf(stderr,"reclaim(%d,%d) pid=%d, reaped=%d, chan=%s, host=%s\n",
+	  fromfd,tofd,proc->pid,proc->reaped,proc->ch->name,proc->ho->name);
+
+	proc->pid = 0;
+	proc->reaped = 0;
+	if (proc->carryover != NULL) {
 	  fprintf(stderr, "%s: HELP! Lost %d bytes: '%s'\n",
-		  progname, strlen(cpids[fromfd].carryover),
-		  cpids[fromfd].carryover);
-	  free(cpids[fromfd].carryover);
-	  cpids[fromfd].carryover = NULL;
+		  progname, strlen(proc->carryover), proc->carryover);
+	  free(proc->carryover);
+	  proc->carryover = NULL;
 	}
-	if (cpids[fromfd].ch != NULL) {
-	  --cpids[fromfd].ch->kids;
-	  if (cpids[fromfd].ch->fifohead)
-	    fifoschedule(cpids[fromfd].ch);
-	  else if (cpids[fromfd].ch->kids == 0 &&
-		   cpids[fromfd].ch->link == NULL) {
-	    unweb(L_CHANNEL, cpids[fromfd].ch);
-	    cpids[fromfd].ch = NULL;
+	if (proc->ch != NULL) {
+	  proc->ch->kids -= 1;
+	  if (proc->ch->kids == 0 && proc->ch->link == NULL) {
+	    unweb(L_CHANNEL, proc->ch);
+	    proc->ch = NULL;
 	  }
 	}
-	if (cpids[fromfd].ho != NULL) {
-	  --cpids[fromfd].ho->kids;
-	  if (cpids[fromfd].ho->fifohead)
-	    fifoschedule(cpids[fromfd].ho);
-	  else if (cpids[fromfd].ho->kids == 0 &&
-		   cpids[fromfd].ho->link == NULL) {
-	    unweb(L_HOST, cpids[fromfd].ho);
-	    cpids[fromfd].ho = NULL;
+	if (proc->ho != NULL) {
+	  proc->ho->kids -= 1;
+	  if (proc->ho->kids == 0 && proc->ho->link == NULL) {
+	    unweb(L_HOST, proc->ho);
+	    proc->ho = NULL;
 	  }
 	}
 	close(fromfd);
@@ -392,8 +320,9 @@ static void reclaim(fromfd, tofd)
 	/* Reschedule the vertices that are left
 	   (that were not reported on).		*/
 
-	nvp = cpids[fromfd].vertices;
-	cpids[fromfd].vertices = NULL;
+	nvp = proc->vertex;
+	proc->vertex = NULL;
+	proc->thread = NULL;
 	for (vp = nvp; vp != NULL ; vp = nvp) {
 	  nvp = vp->nextitem;
 	  if (vp->ngroup == 0)
@@ -405,60 +334,13 @@ static void reclaim(fromfd, tofd)
 static void waitandclose(fd)
 	int	fd;
 {
-#ifndef USE_SIGREAPER
-	int	n, pid;
-
-#if 0
-	/* This is a quick and dirty hack (for A/UX I think). BEWARE */
-	if (cpids[fd].pid <= 0) {
-	  fprintf(stderr,"fd %d: pid is %d ??\n",fd,cpids[fd].pid);
-	  return;
-	}
-	n = waitpid(cpids[fd].pid, (int *)NULL, 0);
-	if(n != cpids[fd].pid && (n != -1 || errno != 10)) {
-	  fprintf(stderr,"fd %d, pid %d: waitpid says %d, errno %d\n",
-		  fd,cpids[fd].pid, n, errno);
-	  return;
-	}
-	--numkids;
-	reclaim(fd, cpids[fd].tofd );
-#else
-	if (cpids[fd].pid > 0) {
-#ifdef	USE_UNIONWAIT
-	  while ((pid = wait((union wait *)0)) > 0)
-#else	/* !USE_UNIONWAIT */
-	  while ((pid = wait((int *)0)) > 0)
-#endif	/* USE_UNIONWAIT */
-	    {
-	      --numkids;
-	      if (verbose)
-		printf("reaped %d\n", pid);
-	      if (pid == cpids[fd].pid) {
-		reclaim(fd, cpids[fd].tofd);
-		break;
-	      } else {
-		for (n =; n < scheduler_nofiles; ++n) {
-		  if (cpids[n].pid == pid) {
-		    cpids[n].pid = -pid;
-		    ++numkids;
-		    break;
-		  }
-		}
-	      }
-	    }
-	} else if (cpids[fd].pid < 0) {
-	  --numkids;
-	  reclaim(fd, cpids[fd].tofd);
-	}
-#endif
-#else	/* USE_SIGREAPER */
 	/* This is called when
 	   - fd return 0 (EOF)
 	   - fd returns -1, and errno != {EAGAIN|EWOULDBLOCK|EINTR}
 	 */
 	--numkids;
 	reclaim(fd, cpids[fd].tofd);
-#endif
+
 }
 
 #ifdef	USE_SELECT
@@ -507,7 +389,6 @@ time_t timeout;
 	struct sockaddr_in raddr;
 	int	raddrlen;
 	struct timeval tv;
-	time_t now;
 
 	time(&now);
 
@@ -556,6 +437,10 @@ time_t timeout;
 	      if (rc == 0) { /* Child! */
 		close(querysocket);
 		qprint(i);
+		/* When running memory leakage debuggers we do not care
+		   that this leaks memory, thus we run /bin/true, and
+		   things like PURIFY and DEC ATOM don't complain.. */
+		execl("/bin/true","true");
 		_exit(0);
 	      }
 	      if (rc > 0)
@@ -582,14 +467,13 @@ void
 queryipcinit()
 {
 #ifdef	AF_INET
-	time_t now;
 	struct servent *serv;
 	struct sockaddr_in sad;
 	int on = 1;
 
 	if (querysocket >= 0)
 		return;
-	now = time((time_t *)0);
+	time(&now);
 	if ((serv = getservbyname("mailq", "tcp")) == NULL) {
 	  fprintf(stderr, "No 'mailq' tcp service defined!\n");
 	  /* try again in 5 minutes or so */
@@ -627,7 +511,6 @@ mux(timeout)
 time_t timeout;
 {
 	int	fd;
-	time_t	now;
 
 	/*
 	 * Nice'n easy and simpleminded: grab a random file descriptor,
@@ -678,6 +561,7 @@ static void readfrom(fd)
 	  cpids[fd].carryover = NULL;
 	  dontbreak = 1;
 	}
+
 	/* Note that if we get an alarm() call, the read will return -1, TG */
 	errno = 0;
 	while ((n = read(fd, dontbreak ? cp : buf,
@@ -701,6 +585,7 @@ static void readfrom(fd)
 	    } else
 	      dontbreak = 1;
 	  }
+
 	  if (dontbreak && cp == buf + bufsize) {
 	    if (pcp == buf) {
 	      /* XX:
@@ -724,12 +609,18 @@ static void readfrom(fd)
 	    break;
 #endif				/* !USE_BSDNODELAY */
 	}
+
 	if (verbose) {
-	  fprintf(stderr, "read from %d returns %d, errno=%d\n", fd, n, errno);
+	  if (!(errno == EAGAIN || errno == EWOULDBLOCK))
+	    fprintf(stderr,
+		    "read from %d returns %d, errno=%d\n", fd, n, errno);
 	}
 	if (n == 0 || (n < 0 && errno != EWOULDBLOCK && errno != EAGAIN &&
 		       errno != EINTR)) {
 	  /*printf("about to call waitandclose(), n=%d, errno=%d\n",n,errno);*/
+	  if (cpids[fd].tofd >= 0)
+	    close(cpids[fd].tofd);
+	  cpids[fd].tofd = -1;
 	  waitandclose(fd);
 	}
 	/* fprintf(stderr, "n = %d, errno = %d\n", n, errno); */
@@ -834,18 +725,17 @@ int signum;
 	int statloc;
 #endif
 #ifdef USE_UNIONWAIT
-	while ((pid = wait3(&statloc,WNOHANG,NULL)) > 0) ;
+	pid = wait3(&statloc,WNOHANG,NULL);
 #else
 #ifdef	USE_WAITPID
-	while ((pid = waitpid(-1,&statloc,WNOHANG)) > 0) ;
+	pid = waitpid(-1,&statloc,WNOHANG);
 #else
 	pid = wait(&statloc);
 #endif
 #endif
+	if (pid <= 0) return;
 
 
-#if 0	/* DO NOT do anything excessive -- we don't care WHO died.. */
-
 #ifdef	WIFEXITED
 	if (WIFEXITED(statloc)) ok = 1;
 #else
@@ -877,7 +767,7 @@ printf(" .. already reaped ??\n");
 	    }
 	  }
 	}
-#endif
+
 	/* re-instantiate the signal handler.. */
 #ifdef SIGCLD
 	SIGNAL_HANDLE(SIGCLD,  sig_chld);
