@@ -462,7 +462,7 @@ checkrouter()
 void
 checkscheduler()
 {
-	int pid, n, tn, r, sawcore;
+	int pid, n, tn, r, sawcore = 0;
 	FILE *fp;
 	struct stat corebuf;
 	struct NDIR_TYPE *dp;
@@ -470,18 +470,19 @@ checkscheduler()
 
 	if (postoffice == NULL)
 	  return;
-	sprintf(path, "%s/%s", postoffice, SCHEDULERDIR);
+
+	sprintf(path, "%s/%s", postoffice, TRANSPORTDIR);
 	if ((dirp = opendir(path)) == NULL) {
 	  fprintf(stderr, "%s: opendir(%s):", progname, path);
 	  perror(" ");
 	  return;
 	}
-	for (dp = readdir(dirp), n = tn = 0; dp != NULL; dp = readdir(dirp)) {
-	  /* beware of directories that fit the pattern... */
+	sawcore = 0;
+	for (dp = readdir(dirp), n = 0; dp != NULL; dp = readdir(dirp)) {
 	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0]))
 	    ++n;
-	  else
-	    ++tn;
+	  else if (strcmp("core",dp->d_name)==0)
+	    sawcore = 1;
 	}
 #ifdef	BUGGY_CLOSEDIR
 	/*
@@ -492,33 +493,24 @@ checkscheduler()
 	close(dirp->dd_fd);
 #endif
 	closedir(dirp);
-	tn -= 2; /* . and .. */
-	printf("%d entr%s in scheduler queue: ", n, n != 1 ? "ies" : "y");
-
-	sprintf(path, "%s/%s/core", postoffice, SCHEDULERDIR);
-	sawcore = (stat(path, &corebuf) == 0);
-	tn -= sawcore;
+	printf("%d message%s in transport queue: ",
+	       n, n != 1 ? "s" : "");
 
 	if (nonlocal)
 	  r = -2;
 	else
 	  r = isalive(PID_SCHEDULER, &pid, &fp);
+
 	switch (r) {
 	case EX_UNAVAILABLE:
-	  printf("no daemon");
+	  printf("no scheduler daemon");
 	  fclose(fp);
 	  break;
 	case EX_OK:
-	  if (n) {
-	    if (tn > 0)
-	      printf("busy");
-	    else
-	      printf("processing control files");
-	  } else if (tn > 0)
-	    printf("scheduling transports");
-	  else
+	  if (n == 0)
 	    printf("idle");
-	  fclose(fp);
+	  else
+	    printf("working");
 	  break;
 	case -2:
 	  printf("non-local");
@@ -533,28 +525,6 @@ checkscheduler()
 	  printf(" (core exists)");
 	printf("\n");
 
-	sprintf(path, "%s/%s", postoffice, TRANSPORTDIR);
-	if ((dirp = opendir(path)) == NULL) {
-	  fprintf(stderr, "%s: opendir(%s):", progname, path);
-	  perror(" ");
-	  return;
-	}
-	for (dp = readdir(dirp), n = 0; dp != NULL; dp = readdir(dirp)) {
-	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0]))
-	    ++n;
-	}
-#ifdef	BUGGY_CLOSEDIR
-	/*
-	 * Major serious bug time here;  some closedir()'s
-	 * free dirp before referring to dirp->dd_fd. GRRR.
-	 * XX: remove this when bug is eradicated from System V's.
-	 */
-	close(dirp->dd_fd);
-#endif
-	closedir(dirp);
-	if (n)
-	  printf("%d message%s in transport queue\n",
-		 n, n != 1 ? "s" : "");
 }
 
 int
