@@ -98,15 +98,29 @@ main(argc, argv)
 	  exit(EX_USAGE);
 	}
 
-	while (!getout && fgets(file, sizeof file, stdin)) {
+	while (!getout) {
+	  char *s;
+
+	  printf("#hungry\n");
+	  fflush(stdout);
+	  if (fgets(file, sizeof file, stdin) == NULL)
+	    break;
+	  /* Input:
+	       spool/file/name [ \t host.info ] \n
+	   */
+
 	  if (emptyline(file, sizeof file))
 	    break;
 
+	  s = strchr(file,'\t');
+	  if (s != NULL)
+	    *s = 0;	/* Ignore the host-selector */
+
 	  dp = ctlopen(file, channel, (char *)NULL, &getout, ctlsticky, NULL);
-	  if (dp == NULL)
-	    continue;
-	  process(dp);
-	  ctlclose(dp);
+	  if (dp != NULL) {
+	    process(dp);
+	    ctlclose(dp);
+	  }
 	}
 	exit(0);
 	/* NOTREACHED */
