@@ -103,12 +103,12 @@ ctlclose(dp)
 	    continue;
 	  diagnostic(rp, EX_TEMPFAIL, "address was left locked!!");
 	}
-	if (dp->ctlfd >= 0)
-	  close(dp->ctlfd);
 #ifdef	USE_MMAP
 	if (dp->let_buffer != NULL)
 	  munmap(dp->let_buffer, dp->let_end - dp->let_buffer);
 #endif
+	if (dp->ctlfd >= 0)
+	  close(dp->ctlfd);
 	if (dp->msgfd >= 0)
 	  close(dp->msgfd);
 	if (dp->contents != NULL)
@@ -198,7 +198,6 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	register char *s;
 	char *mfpath;
 	int i, n;
-	static struct ctldesc d;
 	struct address *ap;
 	struct rcpt *rp;
 	struct stat stbuf;
@@ -208,6 +207,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	int headers_spc = 0;
 	int largest_headersize = 80; /* Some magic minimum.. */
 
+	static struct ctldesc d; /* ONLY ONE OPEN AT THE TIME! */
+
+	memset(&d,0,sizeof(d));
+
 	if ((d.ctlfd = open(file, O_RDWR, 0)) < 0) {
 	  char cwd[MAXPATHLEN], buf[MAXPATHLEN+MAXPATHLEN+100];
 	  int e = errno;	/* Save it over the getwd() */
