@@ -75,28 +75,28 @@ notaryreport(arg1,arg2,arg3)
 
 /*VARARGS*/
 void
-diagnostic(va_alist) /* (rp, rc, "fmtstr", remotemsg) */
+diagnostic(rp, rc, fmt, va_alist) /* (rp, rc, "fmtstr", remotemsg) */
+	struct rcpt *rp;
+	int rc;
+	char *fmt;
 	va_dcl
 {
-	struct rcpt *rp;
 	char	mark, message[8192];
 	register char *cp, *s, *es, *s2;
 	va_list	ap;
 
-	va_start(ap);
-	rp = va_arg(ap, struct rcpt *);		/* arg1: rp */
-	rp->status = va_arg(ap, int);		/* arg2: rc */
+	rp->status = rc;
 
-	cp = va_arg(ap, char *);		/* arg3: fmtstr */
+	va_start(ap);
 	es = &message[sizeof message - 2];
-	for (s = message; cp != NULL && *cp != '\0'; ++cp) {
+	for (s = message; fmt != NULL && *fmt != '\0'; ++fmt) {
 		if (s >= es)
 			break;
-		if (*cp != '%') {
-			*s++ = *cp;
+		if (*fmt != '%') {
+			*s++ = *fmt;
 			continue;
 		}
-		switch (*++cp) {
+		switch (*++fmt) {
 		case 's':	/* string */
 			for (s2 = va_arg(ap, char *); *s2 != 0; ++s2) {
 				*s++ = *s2;
@@ -114,7 +114,7 @@ diagnostic(va_alist) /* (rp, rc, "fmtstr", remotemsg) */
 			*s++ = '%';
 			break;
 		case '\0':
-			--cp;	/* exit soonest */
+			--fmt;	/* exit soonest */
 			break;
 		}
 	}
