@@ -185,12 +185,13 @@ int readalready = 0;		/* does buffer contain valid message data? */
 int wantreserved = 0;		/* open connection on secure (reserved) port */
 int statusreport = 0;		/* put status reports on the command line */
 int noMX = 0;			/* do not use MX records to deliver mail */
-int firstmx = 0;		/* [Matti Aarnio] error in smtpwrite("HELO".. */
+int firstmx = 0;		/* [Matti Aarnio] error in smtpwrite("HELO"..*/
 int mxcount = 0;
 int force_8bit = 0;		/* Claim to the remote to be 8-bit system, even
 				   when it doesn't report itself as such..*/
 int force_7bit = 0;		/* and reverse the previous.. */
 int checkwks = 0;
+int headers_contain_8bit = 0;
 FILE *logfp = NULL;
 extern int nobody;
 int daemon_uid = -1;
@@ -219,7 +220,8 @@ int D_alloc = 0;		/* Memory usage debug */
 
 int  ehlo_capabilities = 0;
 long ehlo_sizeval      = 0;
-char esmtp_on_banner = -1;
+int  esmtp_on_banner = -1;
+int  main_esmtp_on_banner = -1;
 
 
 extern int errno;
@@ -295,6 +297,7 @@ main(argc, argv)
 	char file[MAXPATHLEN+128];
 	char *channel = NULL, *host = NULL;
 	int i, fd, errflg, c, smtpstatus;
+	int need_host = 0;
 	FILE *smtpfp[2];
 	struct ctldesc *dp;
 #ifdef	BIND
@@ -343,7 +346,7 @@ main(argc, argv)
 	while ((c = getopt(argc, argv, "c:deh:l:rsxDET:VW78")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
-	    channel = strsave(optarg);
+	    channel = strdup(optarg);
 	    break;
 	  case 'd':		/* turn on debugging output */
 	    ++debug;
@@ -362,7 +365,7 @@ main(argc, argv)
 	    strcpy(myhostname, optarg);
 	    break;
 	  case 'l':		/* log file */
-	    logfile = strsave(optarg);
+	    logfile = strdup(optarg);
 	    break;
 	  case 'r':		/* use reserved port for SMTP connection */
 	    wantreserved = 1;
@@ -378,7 +381,7 @@ main(argc, argv)
 	    break;
 	  case 'E':		/* don't do EHLO, unless target system
 				   has "ESMTP" on its banner */
-	    esmtp_on_banner = 0;
+	    main_esmtp_on_banner = 0;
 	    break;
 	  case 'T':		/* specify Timeout in seconds */
 	    if ((timeout = atoi(optarg)) < 5) {
@@ -407,14 +410,15 @@ main(argc, argv)
 	    break;
 	  }
 	}
-	if (errflg || optind >= argc) {
+	if (errflg || optind > argc) {
 	  fprintf(stderr,
 		  "Usage: %s [-c channel] [host]\n", argv[0]);
 	  exit(EX_USAGE);
 	}
 	if (optind < argc) {
-	  host = strsave(argv[optind]);
-	}
+	  host = strdup(argv[optind]);
+	} else
+	  need_host = 1;
 
 	if (myhostname[0] == 0) {
 	  /* Default it only when not having an explicite value
@@ -474,11 +478,16 @@ main(argc, argv)
 	      }
 	    }
 	    if (host) free(host);
-	    host = strsave(s);
-	  }
+	    host = strdup(s);
+	  } else
+	    if (need_host) {
+	      printf("# smtp needs defined host!\n");
+	      fflush(stdout);
+	      continue;
+	    }
 
 	  if (debug > 1) { /* "DBGdiag:"-output */
-	    fprintf(stdout, "# (fdcnt=%d, file:%s, host:%s)\n", countfds(), file, host);
+	    printf("# (fdcnt=%d, file:%s, host:%s)\n", countfds(), file, host);
 	    fflush(stdout);
 	  }
 		       
@@ -492,6 +501,8 @@ main(argc, argv)
 	  if (dp == NULL)
 	    continue;
 
+	  headers_contain_8bit = 0;
+
 	  if (verboselog) {
 	    fclose(verboselog);
 	    verboselog = NULL;
@@ -508,7 +519,8 @@ main(argc, argv)
 	    smtpstatus = smtpopen(host, smtpfp, verboselog);
 	    if (smtpstatus == EX_OK)
 	      smtp_isopen = 1;
-	  }
+	  } else
+	    smtpstatus = EX_OK; /* Already open, we have a new job file */
 
 	  smtpstatus = process(dp, smtpstatus, host, smtpfp, verboselog);
 	  ctlclose(dp);
@@ -614,6 +626,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	char SMTPbuf[2000];
 	char *s;
 	int conv_prohibit = check_conv_prohibit(startrp);
+	int hdr_mime2 = 0;
 
 	convertmode = _CONVERT_NONE;
 	if (conv_prohibit >= 0) {
@@ -624,6 +637,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  /* If the header says '8BIT' and ISO-8859-* something,
 	     but body is plain 7-bit, turn it to '7BIT', and US-ASCII */
 	  ascii_clean = check_7bit_cleanness(dp);
+
 	  if (ascii_clean && content_kind == 8) {
 	    if (downgrade_charset(startrp, verboselog))
 	      content_kind = 7;
@@ -669,6 +683,12 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    /* ???? This should NOT happen! */
 	    break;
 	  } /* switch().. */
+
+	  hdr_mime2 = headers_need_mime2(startrp);
+	  if (hdr_mime2) {
+	    headers_to_mime2(startrp,verboselog);
+	  }
+
 	}
 
 	if (strcmp(startrp->addr->link->channel,"error")==0)
@@ -729,9 +749,10 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    }
 	  }
 	  
+	  notaryreport(rp->addr->user, NULL, NULL);
 	  r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf);
 	  if (r != EX_OK) {
-	    notaryreport(rp->addr->user, notaryacct(r,"*NOTOK"),NULL);
+	    notaryreport(NULL, notaryacct(r,"*NOTOK"),NULL);
 	    diagnostic(rp, r, "%s", remotemsg);
 	    if (!smtp_isopen)
 	      break;
@@ -880,6 +901,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 
 	writebuf(fp, (char *)NULL, 0);	/* magic init. */
 
+	fflush(fp);
 	alarm(timeout);
 	gotalarm = 0;
 
@@ -1076,6 +1098,7 @@ writebuf(fp, buf, len)
 
 	  if (--alarmcnt <= 0) {
 	    alarmcnt = ALARM_BLOCKSIZE;
+	    fflush(fp);
 	    alarm(timeout);	/* re-arm it */
 	  }
 
@@ -1161,6 +1184,7 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 
 	  if (--alarmcnt <= 0) {
 	    alarmcnt = ALARM_BLOCKSIZE;
+	    fflush(fp);
 	    alarm(timeout);
 	  }
 
@@ -1314,6 +1338,11 @@ smtpopen(host, fparr, verboselog)
 	firstmx = 0;
 	mxcount = 0;
 	do {
+
+	  esmtp_on_banner = main_esmtp_on_banner;
+	  ehlo_capabilities = 0;
+	  ehlo_sizeval = 0;
+
 	  if ((i = smtpconn(host, fparr, verboselog)) != EX_OK)
 	    return i;
 	  if (esmtp_on_banner) {
@@ -1345,6 +1374,19 @@ smtpopen(host, fparr, verboselog)
 		fprintf(logfp, "%d#\t(closed SMTP channel - HELO failed??)\n", pid);
 	      smtp_isopen = 0;
 	    }
+	    if (i == EX_TEMPFAIL) {
+	      /* Ok, sometimes EHLO+HELO cause crash, open and do HELO only */
+	      if ((i = smtpconn(host, fparr, verboselog)) != EX_OK)
+		return i;
+	      i = smtpwrite(fparr, verboselog, 1, SMTPbuf);
+	      if (i == EX_TEMPFAIL && smtp_isopen) {
+		fclose(fparr[0]); fparr[0] = NULL;
+		fclose(fparr[1]); fparr[1] = NULL;
+		if (logfp)
+		  fprintf(logfp, "%d#\t(closed SMTP channel - HELO failed(2))\n", pid);
+		smtp_isopen = 0;
+	      }
+	    }
 	  }
 	} while (i == EX_TEMPFAIL && firstmx <= mxcount);
 	if (debug) {
@@ -1778,22 +1820,22 @@ rmsgappend(va_alist)
 	va_dcl
 {
 	va_list ap;
-	char *s, *arg;
+	char *fmt, *arg;
 	char *cp = remotemsg + strlen(remotemsg);
 	char *cpend = remotemsg + sizeof remotemsg -1;
 
 	va_start(ap);
-	s   = va_arg(ap, char *);
+	fmt = va_arg(ap, char *);
 
-	if (!s) s="(NULL)";
-	for (; *s != 0; ++s) {
-	  if (*s == '%' && *++s == 's') {
+	if (!fmt) fmt="(NULL)";
+	for (; *fmt != 0; ++fmt) {
+	  if (*fmt == '%' && *++fmt == 's') {
 	    arg = va_arg(ap, char *);
 	    while (*arg && cp < cpend)
 	      *cp++ = *arg++;
 	  } else
 	    if (cp < cpend)
-	      *cp++ = *s;
+	      *cp++ = *fmt;
 	}
 	*cp = 0;
 	va_end(ap);
@@ -2087,7 +2129,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	case 551: /* User not local; please try <forward-path> */
 		return EX_NOUSER;
 	case 552: /* Requested mail action aborted: exceeded storage allocation */
-		return EX_TEMPFAIL;
+		return EX_UNAVAILABLE;
 	case 553: /* Requested action not taken: mailbox name not allowed */
 		return EX_NOUSER;
 	case 554: /* Transaction failed */
@@ -2722,10 +2764,7 @@ char *smtpline;
 #ifdef USE_ALLOCA
 	char *str = alloca(len);
 #else
-	char *str = malloc(len);	/* This leaks memory, but we
-					   propably can allow it for
-					   this ephemerally existing
-					   transporter.. */
+	char *str = malloc(len);
 #endif
 	char *s = str;
 
@@ -2735,6 +2774,9 @@ char *smtpline;
 	*s++ = ' ';
 	if (len < 11) {
 	  notaryreport(NULL,NULL,str);
+#ifndef USE_ALLOCA
+	  free(str);
+#endif
 	  return;
 	}
 	*s++ = '(';
@@ -2754,6 +2796,9 @@ char *smtpline;
 	*s++ = ')';
 	*s = 0;
 	notaryreport(NULL,NULL,str);
+#ifndef USE_ALLOCA
+	free(str);
+#endif
 }
 
 
